/**
 * Generates md5 checksums for files. Requires openssl
 * be properly installed on linux.
 *
 * Author: Matt England (moenglan@umich.edu)
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <errno.h>
#ifdef USE_OPENSSL
#include <openssl/md5.h>
#else
#include <md5.h>
#endif

#define CHUNK_SIZE 16384

#ifndef MD5_DIGEST_LENGTH
#define MD5_DIGEST_LENGTH 16
#endif

/**
 * Compute a file's digest and
 * print it using hex notation
 * on stdout.
 *
 * On error, terminate process.
 */
void print_digest(char * filename) {
  FILE * file;
  MD5_CTX ctx;
  unsigned char dig[MD5_DIGEST_LENGTH];
  char chunk[CHUNK_SIZE];
  int numread, i;

  file = fopen(filename, "r");
  if (!file) {
    perror("fopen");
    exit(1);
  }

  MD5Init(&ctx);

  while ((numread = fread(chunk,1,CHUNK_SIZE,file)) > 0) {
    MD5Update(&ctx,chunk,numread);  
  }
  fclose(file);
  if (numread < 0) {
    perror("fread");
    exit(1);
  }
  MD5Final(dig,&ctx);
  for (i = 0; i < MD5_DIGEST_LENGTH; i++) {
    printf("%hhx",dig[i]);
  }
  printf("\t%s\n",filename);
}

int main(int argc, char ** argv) {
  int i;
  if (argc < 2) {
    fprintf(stderr,"usage: %s [files]\n",argv[0]);
  }
  for (i = 1; i < argc; i++) {
    print_digest(argv[i]);
  }
  exit(0);
}
